/*
   btronfs.h
   Copyright (C) 2001-2002 Tomohide Naniwa. All rights reserved.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Last modified: Feb. 6, 2002.
*/

#define BLOCK_SIZE 1024
#define RECORD_INDEX_OFFSET 384

typedef struct  {
  short int disk_id;
  short int disk_type;
  short int block_size;
  short int id_tbl_size;
  short int short_name_tbl_size;
  short int bitmap_size;
  char reserved[12];
  short int logical_block_size;
  short int max_file;
  short int lang;
  short int access_level;
  long int logical_block;
  long int empty_block;
  long int modified_date;
  long int created_date;
  union {
    unsigned char schar[40];
    unsigned short int wchar[20];
  } file_system, device_name;
} SYSTEM_HEADER;

typedef struct {
  long int header_start_id;
  short int file_type;
  short int application_type;
  union {
    unsigned char schar[28];
    unsigned short int wchar[14];
  } owner_name, group_name;
  short int group_access_level;
  short int public_access_level;
  short int link_number;
  short int index_level;
  long int byte;
  long int logical_block;
  long int record;
  char reserved1[8];
  long int keep_time;
  long int access_date;
  long int modified_date;
  long int create_date;
  union {
    unsigned char schar[40];
    unsigned short int wchar[20];
  } file_name;
  union {
    unsigned char schar[24];
    unsigned short int wchar[12];
  } keyword;
  long int file_reference_count;
  char reserved2[10];
  short int file_id;
  long int file_end_id;
} FILE_HEADER;

typedef union {
  char byte[3];
  struct {
    unsigned char low;
    unsigned char mid;
    unsigned char high;
  } address;
} ADDRESS;

typedef union {
  unsigned char count;
  struct {
    unsigned char count;
    unsigned char record_type;
    unsigned short int record_sub_type;
    short int reserve;
    unsigned short int offset;
    unsigned long int record_size;
    unsigned char continue_count;
    ADDRESS logical_block;
  } normal;
  struct {
    struct {
      unsigned char count;
      ADDRESS logical_block;
    } data[4];
  } connect;
  struct {
    unsigned char count;
    unsigned char record_type;
    unsigned short int record_sub_type;
    unsigned short int file_id;
    unsigned short int data[5];
  } link;
} RECORD_INDEX;

typedef struct {
  ADDRESS start_block;
  unsigned char reference_count;
} FILE_ID_TABLE;
