/*
 * TAD data types
 * made from BTRON3 Specification 
 * but extended for using with Semi(?)TAD
 */

#define RT_LINK     0x80
#define RT_TADDATA  0x81
#define RT_TADCMT   0x82
#define RT_TADSUB   0x83
#define RT_TADRSV   0x84
#define RT_SFUSEN   0x85
#define RT_DFUSEN   0x86
#define RT_FFUSEN   0x87
#define RT_MFUSEN   0x88
#define RT_PROG     0x89
#define RT_DATABOX  0x8A
#define RT_FONT     0x8B
#define RT_DICT     0x8C
#define RT_SYSRSV1  0x8D
#define RT_SYSRSV2  0x8E
#define RT_SYSDATA  0x8F

#define TC_NULL 0x0000
#define TC_TAB  0x0009
#define TC_NL   0x000A
#define TC_NC   0x000B
#define TC_FF   0x000C
#define TC_CR   0x000D
#define TC_SP   0x0020
#define TC_LANG 0xFE00
#define TC_SPEC 0xFF00
#define TC_ESC  0xFF80

#define TST_INFO     0xE0
#define TST_TEXT     0xE1
#define TST_TEXTEND  0xE2
#define TST_FIG      0xE3
#define TST_FIGEND   0xE4
#define TST_IMAGE    0xE5
#define TST_VOBJ     0xE6
#define TST_DFUSEN   0xE7
#define TST_FFUSEN   0xE8
#define TST_SFUSEN   0xE9

#define TST_TPAGE    0xA0
#define TST_TRULER   0xA1
#define TST_TFONT    0xA2
#define TST_TCHAR    0xA3
#define TST_TATTR    0xA4
#define TST_TSTYLE   0xA5
#define TST_TVAR     0xAD
#define TST_TMEMO    0xAE
#define TST_TAPPL    0xAF

#define TST_FPRIM    0xB0
#define TST_FDEF     0xB1
#define TST_FGRP     0xB2
#define TST_FMAC     0xB3
#define TST_FATTR    0xB4
#define TST_FPAGE    0xB5
#define TST_FMEMO    0xBE
#define TST_FAPPL    0xBF

typedef char			B;
typedef unsigned char		UB;
typedef short int		H;
typedef unsigned short int	UH;
typedef long int		W;
typedef unsigned long int	UW;

typedef struct {
 H   h;
 H   v;
} PNT;

typedef struct {
H   left;
H   top;
H   right;
H   bottom;
} RECT;

typedef UH  TC;
typedef UW  COLOR;
typedef H   UNITS;
typedef UH  CHSIZE;
typedef UH  SCALE;
typedef UH  RATIO;

typedef struct  {
    UH  id;
    UH  len;
} TSS_GEN;

typedef struct {
    TSS_GEN header; 
    UH  subid;
    UH  sublen;
    UH  data[0];
} TSS_INFO;

typedef struct {
    TSS_GEN header; 
    RECT    view;
    RECT    draw;
    UNITS   h_unit;
    UNITS   v_unit;
    UH      lang;
    UH      bgpat;
} TSS_TEXT;

typedef struct {
    TSS_GEN header; 
    RECT    view;
    RECT    draw;
    UNITS   h_unit;
    UNITS   v_unit;
    W   ratio;
} TSS_FIG;

typedef struct {
    TSS_GEN header; 
    RECT    view;
    RECT    draw;
    UNITS   h_unit;
    UNITS   v_unit;
    H       slope;
    UH      color;
    UH      cinfo[4];
    UW      extlen;
    UW      extend;
    UW      mask;
    H       compac;
    H       planes;
    H       pixbits;
    H       rowbytes;
    RECT    bounds;
    UW  base_off[0];
} TSS_IMAGE;

typedef struct {
    TSS_GEN header; 
    RECT    view;
    H       height;
    CHSIZE  chsz;
    COLOR   frcol;
    COLOR   chcol;
    COLOR   tbcol;
    COLOR   bgcol;
    UH      dlen;
    UB	    data[0];
} TSS_VOBJ;

typedef struct {
    TSS_GEN header; 
    RECT    view;
    CHSIZE  chsz;
    COLOR   frcol;
    COLOR   chcol;
    COLOR   tbcol;
    UH      pict;
    UH      appl[3];
    union {
      UB    schar[32];
      UH    wchar[16];
    } name;
    union {
      UB    schar[32];
      UH    wchar[16];
    } dtype;
    UH      dlen;
    UB      data[0];
} TSS_FFUSEN;

typedef struct {
    TSS_GEN header; 
    RECT    view;
    CHSIZE  chsz;
    COLOR   frcol;
    COLOR   chcol;
    COLOR   tbcol;
    UH      pict;
    UH      appl[3];
    union {
      UB    schar[32];
      UH    wchar[16];
    } name;
    UW	    dlen;
    UB      data[0];
} TSS_DFUSEN;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
    union {
      struct {
	UH      length;
	UH      width;
	UH      top;
	UH      bottom;
	UH      left;
	UH      right;
      } page;
      struct {
	UH      top;
        UH      bottom;
        UH      left;
        UH      right;
      } margine;
      struct {
	UH      colsp;
	UH      colline;
      } column;
      struct {
	UB      data[0];
      } overlaydef;
      struct {
	UH      overlay;
      } overlayspec;
      struct {
	RECT    area;
      } frame;
      struct {
	UH      num;
      } pagenumber;
      struct {
	SCALE   remain;
      } newpage;
    } body;
} TSS_TPAGE;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
    union {
      struct {
	SCALE pitch;
      } pitch;
      struct {
	SCALE   height;
        SCALE   pargap;
        H       left;
        H       right;
        H       indent;
        H       ntabs;
        UH      tabs[0];
      } tab;
      struct {
	SCALE   height;
        SCALE   pargap;
        UH      line;
        H       nfld;
	struct {
	  UH    fld;
	  UH    left;
	  UH    right;
	  UH    margin;
	  UH    f_attr;
	} data[0];
      } field;
    } body;
} TSS_TRULER;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
    union {
      struct {
	UH    class;
	UB    name[0];
      } fontspec;
      struct {
	UH  attr;
      } attr;
      struct {
	CHSIZE  size;
      } size;
      struct {
	RATIO   h_ratio;
        RATIO   w_ratio;
      } scale;
      struct {
	UH  angle;
      } rotate;
      struct {
	COLOR  color;
      } color;
      struct {
	SCALE   base;
      } basemove;
    } body;
} TSS_TFONT;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_TUB;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_TATTR;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_TSTYLE;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_TVAR;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_TMEMO;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_TAPPL;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FPRIM;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FDEF;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FGRP;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FMAC;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FATTR;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FPAGE;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FMEMO;

typedef struct {
    TSS_GEN header;
    UH      subid_attr;
} TSS_FAPPL;
